/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 *   THIS PLUGIN (C) BY JOHANNES KNABE, UNIVERSITY OF HERTFORDSHIRE,UK   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef GRNCELLS_H
#define GRNCELLS_H

#include <CompuCell3D/Steppable.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Potts3D/CellGChangeWatcher.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <BasicUtils/BasicClassGroup.h> //had to include it to avoid problems with template instantiation
#include <CompuCell3D/Field3D/AdjacentNeighbor.h>
#include <CompuCell3D/dllDeclarationSpecifier.h>
#include <CompuCell3D/Potts3D/CellGChangeWatcher.h>
#include "./KnabeGRN.h"

#include <map>
#include <vector>
#include <set>


#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Field3D/Dim3D.h>
#include <CompuCell3D/plugins/CellType/CellTypePlugin.h>
#include <CompuCell3D/plugins/ContactMultiCad/ContactMultiCadEnergy.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <CompuCell3D/plugins/ContactMultiCad/ContactMultiCadData.h>

using namespace CompuCell3D;

//define EXP_STL
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include "GRNcells.h"

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/steppables/GRNcells/GRNconcentrationsData.h>
//include "./GRNconcentrationsData.h"

#include <CompuCell3D/Potts3D/Cell.h>
#include <PublicUtilities/StringUtils.h>

#include <string>
#include <sstream>
#include <fstream>
#include <map>
#include <cmath>
#include <iostream>
#include <algorithm>




namespace CompuCell3D {
  class Potts3D;
  class Simulator;
  class Dim3D;
  class BoundaryStrategy;
  class CellTypePlugin;
  class Cell;
  class Field3DIndex;
  class GRNconcentrationsData;

  template <class T> class Field3D;
  class CellInventory;

  class GRNcells : public Steppable, public CellGChangeWatcher  {
    Potts3D *potts;
    Simulator *simulator;
	KnabeGRN *GRN;


  public:
	bool findExtremePoints(double angle, Point3D extra, vector<Point3D>::iterator begin, vector<Point3D>::iterator end, Point3D &Vmin, Point3D &Vmin2nd, Point3D &Vmax, Point3D &Vmax2nd);

	BasicClassAccessor<GRNconcentrationsData> GRNconcentrationsDataAccessor;

	BasicClassAccessor<GRNconcentrationsData> * getGRNconcentrationsDataAccessorPtr(){
	  return & GRNconcentrationsDataAccessor;
	}

    // BCGChangeWatcher interface
    virtual void field3DChange(const Point3D &pt, CellG *newCell,
                                CellG *oldCell);

    GRNcells();
	//~GRNcells();

    void setPotts(Potts3D *potts) {this->potts = potts;}

	bool findCellPoint(CellG* cell, Point3D &cm, double depth);
	void getCellPixels(CellG* cell, Point3D &cm, std::vector<Point3D>& cP);
	float getAvgConcentration(std::vector<Point3D>& pixels, std::string field);
	void changeConcentration(std::vector<Point3D>& pixels, std::string field, float changeConc);
	void setAvgConcentration(std::vector<Point3D>& pixels, std::string field, float avgConc);

    // SimObject interface
    virtual void init(Simulator *simulator);

    // Begin Steppable interface
    virtual void start();
    virtual void step(const unsigned int currentStep);
    virtual void finish();
    // End Steppable interface

    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
  };
};
#endif
