#ifndef COMPUCELL3DDIFFUSIONSOLVERFE_H
#define COMPUCELL3DDIFFUSIONSOLVERFE_H


#include <CompuCell3D/Steppable.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include "DiffusableVector.h"

#include "DiffSecrData.h"

#include <string>

#include <vector>
#include <set>
#include <map>
#include <iostream>
#include <CompuCell3D/Field3D/Array3D.h>

namespace CompuCell3D {

/**
@author m
*/
//forward declarations
class Potts3D;
class Simulator;
class Cell;
class CellInventory;
class Automaton;

class DiffusionData;
class SecretionDataFlex;
class BoundaryStrategy;

template <typename Y> class Field3D;
template <typename Y> class Field3DImpl;

class DiffusionSolverFE:public Steppable
{
  public :
   typedef void (DiffusionSolverFE::*diffSecrFcnPtr_t)(void);
   typedef void (DiffusionSolverFE::*secrSingleFieldFcnPtr_t)(unsigned int);
   typedef float precision_t;
   typedef Array3DBorders<precision_t>::ContainerType Array3D_t;
   typedef Array3DBordersField3DAdapter<precision_t> ConcentrationField_t;
   
protected:

   Potts3D *potts;
   Simulator *simPtr;
   BoundaryStrategy * boundaryStrategy;
   
   unsigned int currentStep;
   unsigned int maxDiffusionZ;
   float diffConst;
   float decayConst;
   float deltaX;///spacing
   float deltaT;///time interval
   float dt_dx2; ///ratio delta_t/delta_x^2
   Field3D<CellG *> *cellFieldG;
   Automaton *automaton;
   
   std::vector<DiffusionData> diffDataVec;
   std::vector<SecretionData> secrDataVec;
   std::vector<ConcentrationField_t * > concentrationFieldVector;
   std::vector<std::string> concentrationFieldNameVector;

   ConcentrationField_t  workField;
   ConcentrationField_t  scratchField;

   unsigned int maxNeighborIndex;
   std::vector<Point3D> offsetVec;
   CellInventory *cellInventoryPtr;
   
   void (DiffusionSolverFE::*diffusePtr)(void);///ptr to member method - Forward Euler diffusion solver
   void (DiffusionSolverFE::*secretePtr)(void);///ptr to member method - Forward Euler diffusion solver
   void diffuse();
   void diffuseSingleField(unsigned int idx);
   void secrete();
   void secreteOnContact();
   void secreteSingleField(unsigned int idx);
   void secreteOnContactSingleField(unsigned int idx);
   void scrarch2Concentration(ConcentrationField_t *scratchField, ConcentrationField_t *concentrationField);
   void initWorkField(ConcentrationField_t *concentrationField,ConcentrationField_t *workField);
   void scrarch2Concentration2(ConcentrationField_t *scratchField, ConcentrationField_t *concentrationField);
   void outputField( std::ostream & _out,Field3DImpl<float> *_concentrationField);
   void readConcentrationField(std::string fileName,Field3DImpl<float> *concentrationField);
   
   unsigned int numberOfFields;
   Dim3D fieldDim;
   Dim3D workFieldDim;

   void initializeConcentration();   
public:
    DiffusionSolverFE();

    virtual ~DiffusionSolverFE();
    
   
    virtual void init(Simulator *simulator);
    virtual void extraInit(Simulator *simulator);
    
    // Begin Steppable interface
    virtual void start();
    virtual void step(const unsigned int _currentStep);
    virtual void finish() {}
    // End Steppable interface

    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out){};
    // End XMLSerializable interface

    
};




};



#endif
